package com.mystudy.net_multi2;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.net.Socket;
import java.net.UnknownHostException;
import java.util.Scanner;

/* ===  채팅 클라이언트 ===
 독립적으로 동작할 쓰레드 만들기 (읽기전용, 쓰기 전용 각각 존재)
   - 쓰기 Thread : 메시지 작성 및 전송을 독립적으로 처리 (언제든지 메시지 보내기 - 쓰기전용)
   - 읽기 Thread : 서버쪽에서 보내온 메시지 읽어서 화면표시 (언제든지 읽기 - 읽기전용)   
 */
public class TCPClientMultiChat {

	public static void main(String[] args) {
		// 송신, 수신 Thread 생성 및 실행
		System.out.println("=== main() 시작");
		
		Scanner scanner = new Scanner(System.in);
		System.out.print("사용할 이름 입력> ");
		String name = scanner.nextLine();
		
		Socket socket = null;
		
		try {
			socket = new Socket("192.168.18.31", 10000);
			
			// 메세지 보내기 Thread 생성
			ClientSender clientSender = new ClientSender(socket, name);
			clientSender.start();
			
			// 메세지 받기 Thread 생성
			ClientReceiver clientReceiver = new ClientReceiver(socket);
			clientReceiver.start();
			
		} catch (UnknownHostException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}

		
		System.out.println("=== main() 끝");
	}
	
	// 메세지 보내기 Thread 생성
	static class ClientSender extends Thread {
		Socket socket;
		String name;
		DataOutputStream out;

		public ClientSender(Socket socket, String name) {
			this.socket = socket;
			this.name = name;
			
			try {
				out = new DataOutputStream(socket.getOutputStream());
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		
		@Override
		public void run() {
			//메세지 작성해서 서버로 전송
			if (out == null) {
				System.out.println("[예외발생] 출력 객체가 없습니다");
				return;
			}
			Scanner scanner = new Scanner(System.in);
			
			try {
				out.writeUTF(name); // 첫번째 메세지 전송 : 이름
				
				while (true) {
					System.out.print("메세지 작성 > ");
					String msg = scanner.nextLine();
					
					out.writeUTF("[" + name + "] " + msg);
				}
				
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		
	}
	
	
	// 메세지 받기 Thread 생성
	static class ClientReceiver extends Thread {
		Socket socket;
		DataInputStream in;
		
		public ClientReceiver(Socket socket) {
			this.socket = socket;
			
			try {
				in = new DataInputStream(socket.getInputStream());
			} catch (IOException e) {
				e.printStackTrace();
			}
			
		}
		
		@Override
		public void run() {
			// 메세지를 받아서 화면 출력(반복)
			try {
				while (true) {
					//String msg = in.readUTF();
					System.out.println(in.readUTF());
				}
				
			} catch (IOException e) {
				e.printStackTrace();
			}
			
		}
	}

}
